%RC
% Model of amplitude and frequency response of an RC circuit, driven by an
% AC source.
%
% LAST UPDATED by Andy French September 2020

function RC

R = 100; %Resistance /ohms
Vin = 3; %Source voltage amplitude /volts
C = [1,2.2,4.7]; %Capacitance values /F

%Overlay experimental data?
overlay_experimental_data = 1;

%Blank axes?
blank_axes = 1;

fmax = 3000; %Max frequency /Hz
N = 2000; %Number of data points
fsize = 18; %Fontsize

%Characteristic frequencies /kHz
fc = 1000./( R*C*2*pi);

%Capacitance legend (including characteristic frequencies).
C_lgnd = {};
for n=1:length(C)
    C_lgnd{n} = ['C=',num2str(C(n)),'\muF, f_c=',num2str(fc(n),3),'kHz'];
end

%Tickmarks for graphs

%Frequencies /Hz
fticks = 0:500:3000; fminorticks = 1:50:3000;

%Currents /mA
Iticks = 0:5:25; Iminorticks = 0:1:25;

%|Vc/Vin| gain
gainticks = 0:0.1:1; gainminorticks = 0:0.02:1;

%|Vc/Vin| phase
phaseticks = 0:10:100; phaseminorticks = 0:2:100;

%Capacitances /muF
Cticks = 0:5:15; Cminorticks = 0:0.5:15;

%

%Get experimental data
if overlay_experimental_data==1;
    data = import_RC_data;
    [Cdata,Idata,Vin_data,fdata] = import_IC_data;
end

%

%Run RC model
f = linspace(0,fmax,N);
[f,w,mod_Vc_by_Vs,phase_Vc_by_Vs,I] = RC_model( f, R, Vin, C );

%

%Plot phase of Vc/Vs vs f
[fig,ax] = cleargridaxes( fticks,phaseticks, fminorticks, phaseminorticks, fsize );
p = plot( f, (180/pi)*phase_Vc_by_Vs,'linewidth',2 );
xlabel('Frequency /Hz'); ylabel('Vc/Vs phase /deg');
title(['Frequency response of RC circuit. |V_{in}|=',num2str(Vin),'V, R = ',num2str(R),'\Omega.']);
legend(p,C_lgnd,'fontsize',fsize,'location','southeast','orientation','vertical');
print(fig, 'RC amp phase.png','-dpng','-r300'); close(fig);

%

%Plot |Vc/Vs| vs f
[fig,ax] = cleargridaxes( fticks,gainticks, fminorticks, gainminorticks, fsize );
if blank_axes==0
    p = plot( f, mod_Vc_by_Vs,'linewidth',2 );
    if overlay_experimental_data == 1;
        plot(  data(1).f_Hz, data(1).Vc/data(1).Vin, 'b+','markersize',12 );
        plot(  data(2).f_Hz, data(2).Vc/data(1).Vin, 'g+','markersize',12 );
        plot(  data(3).f_Hz, data(3).Vc/data(1).Vin, 'r+','markersize',12 );
    end
    legend(p,C_lgnd,'fontsize',fsize,'location','northeast','orientation','vertical');
end
xlabel('Frequency /Hz'); ylabel('|V_c/V_{in}| voltage gain');
title(['Frequency response of RC circuit. |V_{in}|=',num2str(Vin),'V, R = ',num2str(R),'\Omega.']);
print(fig,'RC gain vs f.png','-dpng','-r300' ); close(fig);

%

%Plot I_RMS vs f
[fig,ax] = cleargridaxes( fticks,Iticks, fminorticks, Iminorticks, fsize );
if blank_axes==0
    p=plot( f, I,'linewidth',2);
    if overlay_experimental_data == 1;
        plot(  data(1).f_Hz, data(1).I_mA, 'b+','markersize',12 );
        plot(  data(2).f_Hz, data(2).I_mA, 'g+','markersize',12 );
        plot(  data(3).f_Hz, data(3).I_mA, 'r+','markersize',12 );
    end
    legend(p,C_lgnd,'fontsize',fsize,'location','southeast','orientation','vertical');
end
xlabel('Frequency /Hz'); ylabel('RMS current /mA');
title(['Frequency response of RC circuit. |V_{in}|=',num2str(Vin),'V, R = ',num2str(R),'\Omega.']);
print(fig, 'RC I.png','-dpng','-r300'); close(fig);

%

%Run RC model for fixed frequency (Hz) and variable capacitance (/muF)
f = 199.9; C = linspace(0,15,N);
[f,w,mod_Vc_by_Vs,phase_Vc_by_Vs,I] = RC_model( f, R, Vin, C );

%Plot I_RMS vs C
[fig,ax] = cleargridaxes( Cticks,Iticks, Cminorticks, Iminorticks, fsize );
if blank_axes==0
    plot( C, I,'b-','linewidth',2 );
    if (overlay_experimental_data == 1)
        plot(Cdata,Idata,'b+','markersize',12);
    end
end
xlabel('Capacitance /\muF'); ylabel('RMS current /mA');  grid on;
title(['RMS current vs capacitance. f=',num2str(f),'Hz. |V_{in}|=',num2str(Vin),'V, R = ',num2str(R),'\Omega.']);
print(fig, 'I vs C.png','-dpng','-r300'); close(fig);

%%

%RC model (f in Hz, R in ohms, Vin in volts, C in muF. Phase output in
%radians, I output in mA.
function [f,w,mod_Vc_by_Vs,phase_Vc_by_Vs,I] = RC_model( f, R, Vin, C )

%Construct input angular frequency and capacitance
N = length(f);
w = 2*pi*f; w = repmat(w.',[1,length(C)]); c = repmat(C,[N,1])*1e-6;

%|Vc/Vs| vs f
mod_Vc_by_Vs = 1./sqrt( 1 + (w*R.*c).^2 );

%Phase of Vc/V0
phase_Vc_by_Vs = atan(w*R.*c );

%Current (RMS) /mA
I = ( 1000/sqrt(2) )*(Vin/R)*w*R.*c./sqrt( 1 + (w.*R.*c).^2 );

%%

%Graph axes with clear grid lines
function [fig,ax] = cleargridaxes( xticks,yticks, xminorticks, yminorticks, fsize )
fig = figure('color',[1 1 1]);
ax = axes('fontsize',fsize,'nextplot','add');
xlim([xticks(1),xticks(end)]);
ylim([yticks(1),yticks(end)]); axis manual; box on;
set(gca,'xtick',xticks,'ytick',yticks);
for n=1:length(xminorticks);
    plot( xminorticks(n)*[1,1], [yticks(1),yticks(end)],'k-' );
end
for n=1:length(yminorticks);
    plot( [xticks(1),xticks(end)], yminorticks(n)*[1,1],'k-' );
end
for n=1:length(xticks);
    plot( xticks(n)*[1,1], [yticks(1),yticks(end)],'k-','linewidth',2 );
end
for n=1:length(yticks);
    plot( [xticks(1),xticks(end)], yticks(n)*[1,1],'k-','linewidth',2 );
end

%%

%Import RC experimental data (f vs I, Vc for fixed V0,R)
function data = import_RC_data
[num,txt,raw] = xlsread( 'RC experiment.xlsx' );
data(1).C = 1.0; data(1).R = 106.7;  data(1).Vin = 3.4;
m = 1;
for n=11:28
    %C=4.7muF
    data(3).f_Hz(m) = raw{n,1};
    data(3).I_mA(m) = raw{n,2};
    data(3).Vc(m) = raw{n,3};
    
    %C=2.2muF
    data(2).f_Hz(m) = raw{n,5};
    data(2).I_mA(m) = raw{n,6};
    data(2).Vc(m) = raw{n,7};
    
    %C=1.0muF
    data(1).f_Hz(m) = raw{n,9};
    data(1).I_mA(m) = raw{n,10};
    data(1).Vc(m) = raw{n,11};
    m = m+1;
end

%%

%Import I /mA vs C /muF data for fixed f, R
function [C,I,Vin,f] = import_IC_data
[num,txt,raw] = xlsread( 'RC experiment.xlsx' );
Vin = raw{5,22}; f = raw{6,22}; m=1;
for n=9:16
    C(m) = raw{n,21}; I(m) = raw{n,22}; m = m+1;
end

%End of code


